@@ -2,7 +2,7 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2016 Nikolaus Rath <Nikolaus@rath.org>
 
-  This program can be distributed under the terms of the GNU GPL.
+  This program can be distributed under the terms of the GNU GPLv2.
   See the file COPYING.
 */
 
@@ -59,7 +59,7 @@
  */
 
 
-#define FUSE_USE_VERSION 31
+#define FUSE_USE_VERSION 34
 
 #include <fuse_lowlevel.h>
 #include <stdio.h>
@@ -76,10 +76,6 @@
    timeout, so we just send a big value */
 #define NO_TIMEOUT 500000
 
-/* We cannot check directly if e.g. O_RDONLY is set, since this is not
- * an individual bit (cf. open(2)) */
-#define ACCESS_MASK (O_RDONLY | O_WRONLY | O_RDWR)
-
 #define MAX_STR_LEN 128
 #define FILE_INO 2
 #define FILE_NAME "current_time"
@@ -224,7 +220,7 @@ static void tfs_open(fuse_req_t req, fuse_ino_t ino,
 
     if (ino == FUSE_ROOT_ID)
         fuse_reply_err(req, EISDIR);
-    else if ((fi->flags & ACCESS_MASK) != O_RDONLY)
+    else if ((fi->flags & O_ACCMODE) != O_RDONLY)
         fuse_reply_err(req, EACCES);
     else if (ino == FILE_INO)
         fuse_reply_open(req, fi);
@@ -243,7 +239,7 @@ static void tfs_read(fuse_req_t req, fuse_ino_t ino, size_t size,
     reply_buf_limited(req, file_contents, file_size, off, size);
 }
 
-static struct fuse_lowlevel_ops tfs_oper = {
+static const struct fuse_lowlevel_ops tfs_oper = {
     .lookup	= tfs_lookup,
     .getattr	= tfs_getattr,
     .readdir	= tfs_readdir,
@@ -293,6 +289,7 @@ int main(int argc, char *argv[]) {
     struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
     struct fuse_session *se;
     struct fuse_cmdline_opts opts;
+    struct fuse_loop_config config;
     pthread_t updater;
     int ret = -1;
 
@@ -344,8 +341,11 @@ int main(int argc, char *argv[]) {
     /* Block until ctrl+c or fusermount -u */
     if (opts.singlethread)
         ret = fuse_session_loop(se);
-    else
-        ret = fuse_session_loop_mt(se, opts.clone_fd);
+    else {
+        config.clone_fd = opts.clone_fd;
+        config.max_idle_threads = opts.max_idle_threads;
+        ret = fuse_session_loop_mt(se, &config);
+    }
 
     fuse_session_unmount(se);
 err_out3:
